@@ -37,8 +37,12 @@ logger = logging.getLogger(__name__)
 REQUIRED_TRANSPORT_FIELDS = {
     "sender_name": lambda t: t.get("sender", {}).get("name", ""),
     "sender_city": lambda t: t.get("sender", {}).get("city", ""),
+    "sender_street": lambda t: t.get("sender", {}).get("street", ""),
+    "sender_zip": lambda t: t.get("sender", {}).get("zip", ""),
     "receiver_name": lambda t: t.get("receiver", {}).get("name", ""),
     "receiver_city": lambda t: t.get("receiver", {}).get("city", ""),
+    "receiver_street": lambda t: t.get("receiver", {}).get("street", ""),
+    "receiver_zip": lambda t: t.get("receiver", {}).get("zip", ""),
     "loading_date": lambda t: bool(t.get("loadingDate")) and all(t.get("loadingDate")),
     "delivery_date": lambda t: bool(t.get("deliveryDate"))
     and all(t.get("deliveryDate")),
@@ -591,3 +595,180 @@ def _sanitize_raw_llm_output(raw_output: str) -> dict[str, Any]:
         except json.JSONDecodeError:
             logger.exception("Error decoding JSON")
     return sanitized_json_data
+
+
+def extract_client_and_overal_from_pdf(
+    text: str,
+    pdf_path: Path,
+    key: str,
+    entities: dict[str, str] | None = None,
+    additional_instructions_client: str = "",
+) -> str | None:
+    logger.info("Extracting entities from PDF...")
+    print("Extracting entities from PDF...")
+    # entities = extract_entities_from_text(text, pdf_path, key)
+
+    logger.info("Extracting client and overall info from PDF...")
+    print("Extracting client and overall info from PDF...")
+    prompts = []
+    prompts.append(
+        return_client_prompt(
+            text,
+            entities.get("Auftraggeber", "") if entities else "",
+            additional_instructions=additional_instructions_client,
+        )
+    )
+    openai.api_key = key
+    pages = convert_pdf_to_image(pdf_path)
+
+    try:
+        json_output: list[dict[str, Any]] = []
+        for prompt in prompts:
+            content_parts: list[
+                ChatCompletionContentPartTextParam | ChatCompletionContentPartImageParam
+            ] = [{"type": "text", "text": prompt}]
+            for path_to_page in pages:
+                base64_image = encode_image(image_path=path_to_page)
+                content_parts.append(
+                    {
+                        "type": "image_url",
+                        "image_url": {"url": f"data:image/jpeg;base64,{base64_image}"},
+                    }
+                )
+
+            messages: list[ChatCompletionMessageParam] = [
+                {"role": "system", "content": "You are a helpful assistant."},
+                {"role": "user", "content": content_parts},
+            ]
+
+            response = openai.chat.completions.create(
+                model="gpt-4o",
+                messages=messages,
+            )
+            raw_output = response.choices[0].message.content
+
+            logger.info("Raw prompt output: %s", raw_output)
+
+            # Delete comments from the code
+            if raw_output:
+                json_output.append(_sanitize_raw_llm_output(raw_output))
+
+        if json_output:
+            result = {
+                "client": json_output[0].get("client", {}),
+                "order_id": json_output[0].get("overall", {}).get("orderID", ""),
+            }
+
+        client_missing, client_available = _check_client_fields(
+            result.get("client", {})
+        )
+        # If the only missing things are client geo-fields, try to infer them
+        geo_fields = {"zip", "city", "country"}
+        if set(client_missing).issubset(geo_fields):
+            _fill_in_missing_for_client(result.get("client", {}), client_available)
+        return json.dumps(result)
+
+    except json.JSONDecodeError:
+        logger.exception("JSON decoding error")
+        return None
+    except Exception:
+        logger.exception("Error generating JSON from text")
+        return None
+
+
+def extract_transports_from_pdf(
+    text: str,
+    pdf_path: Path,
+    client_dict: dict[str, Any],
+    key: str,
+    entities: dict[str, str] | None = None,
+    additional_instructions_transports: str = "",
+) -> str | None:
+    """Extracts transport information from a PDF file using an LLM.
+
+
+    Parameters
+    ----------
+    text : str
+        _description_
+    pdf_path : Path
+        _description_
+    client_dict : dict[str, Any]
+        _description_
+    key : str
+        _description_
+    additional_instructions_transports : str, optional
+        _description_, by default ""
+
+    Returns
+    -------
+    str | None
+        _description_
+    """
+    logger.info("Extracting entities from PDF...")
+    # entities = extract_entities_from_text(text, pdf_path, key)
+
+    logger.info("Extracting transport info from PDF...")
+    prompts = []
+    prompts.append(
+        return_transports_prompt(
+            text,
+            entities if entities else {},
+            additional_instructions=additional_instructions_transports,
+        )
+    )
+    openai.api_key = key
+    pages = convert_pdf_to_image(pdf_path)
+
+    try:
+        json_output: list[dict[str, Any]] = []
+        for prompt in prompts:
+            content_parts: list[
+                ChatCompletionContentPartTextParam | ChatCompletionContentPartImageParam
+            ] = [{"type": "text", "text": prompt}]
+            for path_to_page in pages:
+                base64_image = encode_image(image_path=path_to_page)
+                content_parts.append(
+                    {
+                        "type": "image_url",
+                        "image_url": {"url": f"data:image/jpeg;base64,{base64_image}"},
+                    }
+                )
+
+            messages: list[ChatCompletionMessageParam] = [
+                {"role": "system", "content": "You are a helpful assistant."},
+                {"role": "user", "content": content_parts},
+            ]
+
+            response = openai.chat.completions.create(
+                model="gpt-4o",
+                messages=messages,
+            )
+            raw_output = response.choices[0].message.content
+
+            logger.info("Raw prompt output: %s", raw_output)
+
+            # Delete comments from the code
+            if raw_output:
+                json_output.append(_sanitize_raw_llm_output(raw_output))
+
+        if json_output:
+            result = {
+                "transports": json_output[0].get("transports", []),
+            }
+        combined_dict = {**result, "client": client_dict}
+        transport_missing = _check_transports(combined_dict)
+        if transport_missing:
+            logger.warning(
+                "Some required transport fields are missing: %s", transport_missing
+            )
+
+        apply_hardcoded_cases(combined_dict)
+
+        return json.dumps(combined_dict)
+    except json.JSONDecodeError:
+        logger.exception("JSON decoding error")
+        return None
+    except Exception:
+        logger.exception("Error generating JSON from text")
+        return None
